@@ -682,6 +682,15 @@ watcher_new (const gchar *current_desktop)
   return watcher;
 }
 
+static void
+muted_log_handler (const char     *log_domain,
+                   GLogLevelFlags  log_level,
+                   const char     *message,
+                   gpointer        data)
+
+{
+}
+
 int
 main (int argc, char **argv)
 {
@@ -702,7 +711,6 @@ main (int argc, char **argv)
         sleep (2);
     }
 
-    current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
     xapp_settings = g_settings_new (STATUS_ICON_SCHEMA);
 
     if (g_settings_get_boolean (xapp_settings, DEBUG_KEY))
@@ -718,7 +726,17 @@ main (int argc, char **argv)
     whitelist = g_settings_get_strv (xapp_settings,
                                      VALID_XDG_DESKTOPS_KEY);
 
-    should_start = g_strv_contains ((const gchar * const *) whitelist, current_desktop);
+    current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
+
+    if (current_desktop != NULL)
+    {
+        should_start = g_strv_contains ((const gchar * const *) whitelist, current_desktop);
+    }
+    else
+    {
+        g_warning ("XDG_CURRENT_DESKTOP not set, unable to check against enabled desktop list. Starting anyway...");
+        should_start = TRUE;
+    }
 
     g_strfreev (whitelist);
     g_clear_object (&xapp_settings);
@@ -731,6 +749,19 @@ main (int argc, char **argv)
         exit(0);
     }
 
+    // libdbusmenu and gtk throw a lot of menu-related warnings for problems we already handle.
+    // They can be noisy in .xsession-errors, however. Redirect them to debug output only.
+    DEBUG (""); // Initialize the DEBUGGING flag
+    if (!DEBUGGING)
+    {
+        g_log_set_handler ("LIBDBUSMENU-GTK", G_LOG_LEVEL_WARNING,
+                           muted_log_handler, NULL);
+        g_log_set_handler ("LIBDBUSMENU-GLIB", G_LOG_LEVEL_WARNING,
+                           muted_log_handler, NULL);
+        g_log_set_handler ("Gtk", G_LOG_LEVEL_CRITICAL,
+                           muted_log_handler, NULL);
+    }
+
     watcher = watcher_new (current_desktop);
 
     status = g_application_run (G_APPLICATION (watcher), argc, argv);
